package acceptance;

import java.util.BitSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import automata.DA;
import jdd.JDDNode;
import jdd.JDDVars;
import prism.NondetModel;

public class AcceptanceTransGeneric<Symbol> extends AcceptanceGeneric implements AcceptanceOmega {

	
	protected Set<AcceptanceEdge<Symbol>> trans;
	protected DA<Symbol, ? extends AcceptanceOmega> da;

	public AcceptanceTransGeneric(ElementType kind,  Set<AcceptanceEdge<Symbol>> trans, DA<Symbol, ? extends AcceptanceOmega> da)
	{
		// Needed for correct syntax
		// TODO: How to discard super Constructor
		super(kind, null);
		this.kind = kind;

		this.trans = trans;
		
		this.da = da;
	}

	public AcceptanceTransGeneric(boolean value, DA<Symbol, ? extends AcceptanceOmega> da)
	{
		super(value);
	}
	
	
	public AcceptanceTransGeneric(ElementType kind, AcceptanceTransGeneric<Symbol> left, AcceptanceTransGeneric<Symbol> right, DA<Symbol, ? extends AcceptanceOmega> da)
	{
		super(kind, left, right);
		this.da = da;
	}
	
	@Override
	public AcceptanceType getType()
	{
		return AcceptanceType.GENERICTRANS;
	}

	@Override
	public int hashCode() 
	{
		switch(getKind()) {
		case TRUE:
			return Integer.MIN_VALUE;
		case FALSE:
			return Integer.MAX_VALUE;
		case INF:
			return getTrans().hashCode();
		case INF_NOT:
			return ~getTrans().hashCode();
		case FIN:
			return (-1)*getTrans().hashCode();
		case FIN_NOT:
			return ~((-1)*getTrans().hashCode());
		case AND:
			return getLeft().hashCode() ^ getRight().hashCode();
		case OR:
			return ~(getLeft().hashCode() ^ getRight().hashCode());
		default:
			throw new UnsupportedOperationException();
		}
	}

	public Set<AcceptanceEdge<Symbol>> getTrans() {
		return trans;
	}

	/**
	 * Returns true, if this equals o
	 */
	@Override
	public boolean equals(Object o)
	{
		if(!(o instanceof AcceptanceTransGeneric)) {
			return false;
		}
		
		@SuppressWarnings("unchecked")
		AcceptanceTransGeneric<Symbol> other = (AcceptanceTransGeneric<Symbol>)o;
		if(other.getKind() != getKind()) {
			return false;
		}
		
		switch(getKind()) {
		case TRUE:
		case FALSE:
			return true;
		case AND:
		case OR:
			return getLeft().equals(other.getLeft()) && getRight().equals(other.getRight());
		case FIN:
		case FIN_NOT:
		case INF:
		case INF_NOT:
			return getStates().equals(other.getStates());
		default:
			throw new UnsupportedOperationException();
		}
	}
	
	private boolean bsccContainsEdge(AcceptanceEdge<Symbol> edge, BitSet bscc)
	{
		// TODO: What do with labeling?
		return bscc.get(edge.getSrc()) &&
						bscc.get(edge.getDest());
	}
	
	@Override
	public boolean isBSCCAccepting(BitSet bscc)
	{
		switch(getKind()) {
		case TRUE: return true;
		case FALSE: return false;
		case AND: return left.isBSCCAccepting(bscc) & right.isBSCCAccepting(bscc);
		case OR: return left.isBSCCAccepting(bscc) | right.isBSCCAccepting(bscc);
		case INF:
			// Search for accepting edge within BSCC
			for(AcceptanceEdge<Symbol> edge : getTrans()) {
				if(bsccContainsEdge(edge, bscc)) {
					return true;
				}
			}
			return false;
		case FIN:
			// Search for Fin-edge in BSCC
			for(AcceptanceEdge<Symbol> edge : getTrans()) {
				if(bsccContainsEdge(edge, bscc)) {
					return false;
				}
			}
			return true;
		case INF_NOT:
		case FIN_NOT:
			// TODO
			throw new UnsupportedOperationException();
		default:
			return false;
		}
	}
	
	protected String edgesToString()
	{
		String result = "[";
		Iterator<AcceptanceEdge<Symbol>> it = trans.iterator();
		if (it.hasNext()) {
			result += it.next();
		}
		while(it.hasNext()) {
			result += "," + it.next();
		}
		return result + "]";
	}
	
	@Override
	public String toString()
	{
		switch (getKind()) {
			case TRUE: return "true";
			case FALSE: return "false";
			case AND: return "(" + left.toString() + ") & (" + right.toString() + ")";
			case OR: return "(" + left.toString() + ") | (" + right.toString() + ")";
 			case INF:
 				return "Inf(" + edgesToString() + ")";
 			case FIN:
 				return "Fin(" + edgesToString() + ")";
 			case INF_NOT:
 				return "Inf(!" + edgesToString()  + ")";
 			case FIN_NOT:
 				return "Fin(!" + edgesToString() + ")";
            default:
                throw new UnsupportedOperationException("Unknown generic acceptance kind");
		}
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public AcceptanceOmegaDD toAcceptanceDD(NondetModel model, Vector<JDDNode> labelDDs, JDDVars daDDRowVars, JDDVars daDDColVars) {
		return new AcceptanceGenericDD(
			this,
			model,
			(DA<BitSet, ? extends AcceptanceOmega>)da,
			labelDDs,
			daDDRowVars,
			daDDColVars);
	}
}