//==============================================================================
//	
//	Copyright (c) 2015-
//	Authors:
//	* Joachim Klein <klein@tcs.inf.tu-dresden.de> (TU Dresden)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================
package common;


/**
 * Exception that can be thrown by a method that is called in a computation
 * to notify of some event to indicate that further computation is not necessary.
 * <br>
 * An example would be a consumer of SCC components that wants to
 * abort early once a 'good' SCC has been found.
 * <br>
 * Optionally, an extra object can be attached.
 */
@SuppressWarnings("serial")
public class AbortComputationException extends Exception
{
	private Object extraData;

	public AbortComputationException()
	{
	}

	public AbortComputationException(Object extraData)
	{
		this.extraData = extraData;
	}

	public Object getExtraData()
	{
		return extraData;
	}
}
