package explicit.satsolving;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Set;

import acceptance.AcceptanceGeneric;
import prism.PrismException;

public class ImplicationGraph {
	
	class Conflict extends Literal
	{

		private Conflict(AcceptanceGeneric acc) throws PrismException {
			super(acc);
		}
		
		public Conflict(Clause clause)
		{
			super(-1, false);
			refClause = clause;
		}
		
		public int hashCode()
		{
			return ~refClause.hashCode();
		}
		
		public boolean equals(Object o)
		{
			if(!(o instanceof Conflict)) {
				return false;
			}
			Conflict other = (Conflict)o;
			
			return other.getRefClause().equals(refClause);
		}
		
	}
	
	private LinkedHashMap<Literal, Set<Literal>> adjencency = new LinkedHashMap<Literal, Set<Literal>>();

	public LinkedHashMap<Literal, Set<Literal>> getAdjencency() {
		return adjencency;
	}
	
	private Literal currentLiteral;

	@SuppressWarnings("unchecked")
	public ImplicationGraph clone()
	{
		ImplicationGraph cp = new ImplicationGraph();
		cp.adjencency = (LinkedHashMap<Literal, Set<Literal>>) adjencency.clone();
		return cp;
	}
	
	/**
	 * Adds a literal to the implication graph
	 * @param literal the literal to add
	 * @param implied indicates, whether the literal was generated by unit rule or pure literal rule
	 */
	public void addLiteral(Literal literal, boolean implied)
	{
		if(implied) {
			if(!adjencency.containsKey(currentLiteral)) {
				adjencency.put(currentLiteral, new LinkedHashSet<>());
			}
			Set<Literal> impliedLiterals = adjencency.get(currentLiteral);
			impliedLiterals.add(literal);
		} else {
			if(!adjencency.containsKey(literal)) {
				adjencency.put(literal, new LinkedHashSet<Literal>());
			}
		}
		currentLiteral = literal;
	}
	
	/**
	 * Removes a literal from the implication graph
	 * @param literal the literal which should be removed
	 */
	public void removeLiteral(Literal literal)
	{
		for(Set<Literal> impliedLiterals : adjencency.values()) {
			impliedLiterals.remove(literal);
		}
		adjencency.remove(literal);
	}
	
	/**
	 * Checks, whether there is a conflict in the implication graph
	 * @return true, if there is a conflict in the implication graph
	 */
	public boolean existsConflict()
	{
		//There should be Conflict implying other things
		assert !adjencency.keySet().stream().
				anyMatch(l -> (l instanceof Conflict));
		for(Set<Literal> impliedLiterals : adjencency.values()) {
			if(impliedLiterals.stream().anyMatch(l -> (l instanceof Conflict))) {
				return true;
			}
		}
		return false;
	}
	
	public Clause conflictAnalysis()
	{
		//TODO: Implement
		return null;
	}
}
