package symbolic.satsolving;

import java.util.ArrayList;
import java.util.List;

import acceptance.AcceptanceGenericDD;
import jdd.JDD;
import jdd.JDDNode;
import jdd.JDDVars;
import prism.ECComputer;
import prism.ECComputerDefault;
import prism.ECConsumerStore;
import prism.NondetModel;
import prism.PrismComponent;
import prism.PrismException;

public class TransLiteral extends AbstractLiteral {

	public TransLiteral(AcceptanceGenericDD acc, PrismComponent mc, NondetModel mdp) throws PrismException {
		super(acc, mc, mdp);
	}

	public TransLiteral(int index, boolean negated) 
	{
		super(index, negated);
	}
	
	public TransLiteral(JDDNode states, boolean inf, PrismComponent mc, NondetModel model)
	{
		super(states, inf, mc, model);
	}

	@Override
	public AbstractLiteral clone() {
		AbstractLiteral literal = null;
		switch(kind) {
		case INF:
			literal = new TransLiteral(getStates(), true, mc, mdp);
			break;
		case FIN:
			literal = new TransLiteral(getStates(), false, mc, mdp);
			break;
		case SLACKVAR:
			literal = new TransLiteral(index, false);
			break;
		case NEGSLACKVAR:
			literal = new TransLiteral(index, true);
			break;
		default:
				
		}
		literal.value = value;
		return literal;
	}

	@Override
	public void cutWithMEC(JDDNode mec)
	{
		if(getKind() == AbstractLiteral.Kind.INF || getKind() == AbstractLiteral.Kind.FIN) {
			JDDVars allDDRowVars = mdp.getAllDDRowVars();
			JDDVars allDDColVars = mdp.getAllDDColVars();
			// Copy mec states to primed states
			JDDNode primedMEC = JDD.PermuteVariables(mec.copy(), allDDRowVars, allDDColVars);
		
			// Collect all transitions, that are contained in the mec, i.e.,
			// that have a starting state in mec and a goal state in mec
			JDDNode transInMEC = JDD.And(mdp.getTransReln().copy(),
					mec.copy(),
					primedMEC);
			// Intersect transitions of this and mec
			setStates(JDD.And(getStates(),
					transInMEC));
			if(getStates().equals(JDD.ZERO)) {
				if(getKind() == AbstractLiteral.Kind.INF) {
					// TODO: Look, whether there should be a Deref here
					// JDD.Deref(getStates());
					setFalse();
				}
				if(getKind() == AbstractLiteral.Kind.FIN) {
					setTrue();
					// TODO: Look, whether there should be a Deref here
					// JDD.Deref(getStates());
					setTrue();
				}
			}
		}
	}

	@Override
	public TransLiteral complement()
	{
		switch(getKind()) {
		case SLACKVAR:
			return new TransLiteral(getIndex(), true);
		case NEGSLACKVAR:
			return new TransLiteral(getIndex(), false);
		case INF:
			return new TransLiteral(getStates(), false, mc, mdp);
		case FIN:
			return new TransLiteral(getStates(), true, mc, mdp);
		default:
			return null;
		}
	}
	
	public NondetModel getModel()
	{
		return mdp;
	}

	@Override
	public List<JDDNode> cutMEC(JDDNode mecs) throws PrismException
	{
		if(getKind() == AbstractLiteral.Kind.FIN) {
			JDDVars allDDRowVars = mdp.getAllDDRowVars();
			JDDVars allDDColVars = mdp.getAllDDColVars();
			// Copy mec states to primed states
			JDDNode primedMEC = JDD.PermuteVariables(mecs.copy(), allDDRowVars, allDDColVars);
		
			// Collect all transitions, that are contained in the mec, i.e.,
			// that have a starting state in mec and a goal state in mec
			JDDNode transInMEC = JDD.And(mdp.getTransReln().copy(),
					mecs.copy(),
					primedMEC);
			
			JDDNode cuttedTrans = JDD.And(transInMEC,
					JDD.Not(getStates().copy()));
			List<JDDNode> result = getMECs(mecs, cuttedTrans, null);
			JDD.Deref(cuttedTrans);
			return result;
		}
		
		//Nothing to do for literal.kind != FIN
		List<JDDNode> id = new ArrayList<>(1);
		id.add(mecs.copy());
		return id;
		
	}

	/**
	 * Compute the MECs within a given state space
	 * @param restrict the state space within the MECs are collected
	 * @param accept collect only MECs, that have states in accept. If null, all MECs are collected
	 * @return List of MECs 
	 * @throws PrismException
	 */
	protected List<JDDNode> getMECs(JDDNode restrict, JDDNode trans, JDDNode accept) throws PrismException 
	{
		ECConsumerStore mecStore = new ECConsumerStore(mc, mdp);
		JDDNode transInMEC01 = JDD.And(mdp.getTrans01().copy(),
						trans.copy());
		ECComputer ecComputer = new ECComputerDefault(mc, 
				mecStore,
				mdp.getReach(),
				trans,
				transInMEC01,
				mdp.getAllDDRowVars(),
				mdp.getAllDDColVars(),
				mdp.getAllDDNondetVars());
		ecComputer.computeMECStates(restrict, accept);
		JDD.Deref(transInMEC01);
		return mecStore.getMECStates();
		
	}

}
