package visualisation;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;

import prism.PrismFileLog;
import common.iterable.IterableBitSet;
import explicit.Model;
import explicit.MDPExplicit;

public class AlgoVisualiser
{
	public Model model;
	public List<JSONAlgorithmStep> steps = new ArrayList<JSONAlgorithmStep>();
	public List<String> cssSections = new ArrayList<String>();

	public final static String urlBase = "http://automata.tools/pmc-vis/res";
	public final static String urlCSS = urlBase + "/pmc-vis-0.3.eda4be9872b4650b.css";
	public final static String urlD3JS = urlBase + "/d3.min.e505e1a6ffa27f61.js";
	public final static String urlPMCVIS = urlBase + "/pmc-vis-0.3.9770eb9628a2eca7.js";

	public AlgoVisualiser(Model model)
	{
		this.model = model;
	}

	public void addCSSSection(String css) {
		cssSections.add(css);
	}

	public void addStep(JSONAlgorithmStep step)
	{
		steps.add(step);
	}

	public void exportToHTML(String filename)
	{
		PrismFileLog out = new PrismFileLog(filename);

		out.println("<!DOCTYPE html>");
		out.println("<head>");
		out.println("<meta charset=\"utf-8\">");
		out.println("<link rel=\"stylesheet\" href=\"" + urlCSS +"\">");
		out.println("<script type=\"text/javascript\" src=\"" + urlD3JS + "\"></script>");
		out.println("<script type=\"text/javascript\" src=\"" + urlPMCVIS + "\"></script>");

		for (String cssSection : cssSections) {
			out.println("<style>");
			out.println(cssSection);
			out.println("</style>");
		}

		out.println("</head>");
		out.println("<body>");
		out.println("<div id=\"area\"></div>");

		out.println("<script>");
		StringBuffer sb = new StringBuffer();
		toJSON(sb);
		out.println(sb);
		out.println("</script>");

		out.println("<script>vis = new Visualise(mdp, steps);</script>");

		out.println("</body>");
		out.println("</html>");

		out.close();
	}

	public void exportToJSON(String filename)
	{
		 StringBuffer sb = new StringBuffer();
		 toJSON(sb);
		 PrismFileLog out = new PrismFileLog(filename);
		 out.println(sb);
		 out.close();
	}

	public void toJSON(StringBuffer sb)
	{
		sb.append(((MDPExplicit)model).exportToJSON("mdp"));
		sb.append("\n");

		sb.append("var steps = [\n");
		for (JSONAlgorithmStep step : steps) {
			step.toJSON(sb);
			sb.append(",\n");
		}

		sb.append("];\n");
	}

	protected String stateSetToJSON(BitSet states)
	{
		String s = "[";
		boolean first = true;
		for (int state : IterableBitSet.getSetBits(states)) {
			if (!first) s+=",";
			first = false;
			s += state;
		}
		s+="]";
		return s;
	}
}
